local MAJOR_VERSION = "LibMapTileData-1.1"
local MINOR_VERSION = type('20121013183249') == 'number' and '20121013183249' or date('%Y%m%d%H%M%S')

local _G = _G

if not _G.LibStub then error(MAJOR_VERSION .. " requires LibStub") end

-- Try to create a new library
local library = _G.LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
-- Abort if we are no new library
if not library then
	return
end

-- Usefull for embedding
library.mixinTargets = library.mixinTargets or {}
local mixins = {'ConvertTileIndexToCoords','ConvertCoordsToTileIndex','MinimapTextureSource'}

function library:Embed(target)
	for _,name in pairs(mixins) do
		target[name] = library[name]
	end
	library.mixinTargets[target] = true
end

-- CONSTANTS
-- If someone reads this line, and knows any better way to do this than measuring points in the world, PLEASE tell me.

local measuredPoints = {
	[14] = { -- Eastern Kingdoms
		{
			contX = 17920.816256239 / 40741.175327834,
			contY = 8951.0177271005 / 27149.795290881,
			tileI = 31.536988868228,
			tileJ = 27.839006326567,
		},
		{ -- Booty Bay, Alliance Gryphon landing.
			contX = 17707.818049432 / 40741.175327834,
			contY = 25649.495554782 / 27149.795290881,
			tileI = 31.131021103703,
			tileJ = 59.131298523202,
		},
	},
	[485] = { -- Northrend
		{ -- Borean Tundra, Valliance Keep
			contX = 0.2301621920,
			contY = 0.7072479641,
			tileI = 22.379,
			tileJ = 27.83,
		},
		{ -- Storm Peaks, North of Ulduar, where Murala hides frozen gnomes.
			contX = 0.5992397550,
			contY = 0.0433076012,
			tileI = 34.65,
			tileJ = 13.09,
		},
	},
	[862] = { -- Pandaria
		{ -- Pearlfin village, right inside that watery orb
			contX = 0.7345850945,
			contY = 0.6672819746,
			tileI = 36.957,
			tileJ = 32.415,
		},
		{ -- Winter's Blossom
			contX = 0.3915554576,
			contY = 0.3410157502,
			tileI = 26.99,
			tileJ = 26.09,
		},
	},
}

local mapConstants = {}

do
	for mapContinentId, data in pairs(measuredPoints) do
		local constants = {}
		local minContX, minContXTileI, maxContX, maxContXTileI = 1, 100, 0, 0
		local minContY, minContYTileJ, maxContY, maxContYTileJ = 1, 100, 0, 0
		for _, p in pairs(data) do
			if p.contX < minContX then
				minContX = p.contX
				minContXTileI = p.tileI
			end
			if p.contX > maxContX then
				maxContX = p.contX
				maxContXTileI = p.tileI
			end
			if p.contY < minContY then
				minContY = p.contY
				minContYTileJ = p.tileJ
			end
			if p.contY > maxContY then
				maxContY = p.contY
				maxContYTileJ = p.tileJ
			end
		end
		--local p1 = data[1]
		--local p2 = data[2]
		local xScaleRaw = (maxContXTileI - minContXTileI) / (maxContX - minContX)
		local yScaleRaw = (maxContYTileJ - minContYTileJ) / (maxContY - minContY)
		constants.xScaleRaw = xScaleRaw --* width
		constants.yScaleRaw = yScaleRaw --* height
		constants.xOffset = maxContXTileI - maxContX * xScaleRaw
		constants.yOffset = maxContYTileJ - maxContY * yScaleRaw
		mapConstants[mapContinentId] = constants
	end
end


-- Set up debug stuff
do
	local libDebugLog = LibStub:GetLibrary('LibDebugLog-1.0',true)
	if libDebugLog then
		libDebugLog:Embed(library)
		library:ToggleDebugLog(true)
	else
		function library:Debug (...)
		end
	end
end

function library:ConvertTileIndexToCoords(continent,i,j)
	error("Not Implemented")
end

--[[
-- @param continentMapId a mapId of the continent
-- @param contX value in [0,1]
-- @param contY value in [0,1]
-- @return tileI fractional tile index
-- @return tileJ fractional tile index
--]]
function library:ConvertCoordsToTileIndex(continentMapId,contX,contY)
	assert(type(continentMapId) ~= 'nil')
	assert(type(contX) ~= 'nil', "no continent x given :(")
	assert(type(contY) ~= 'nil')

	if mapConstants[continentMapId] then
		local constants = mapConstants[continentMapId]
		local fi = constants.xOffset + contX * constants.xScaleRaw
		local fj = constants.yOffset + contY * constants.yScaleRaw
		--library:Debug(string.format("cMI=%d, x=%.10f, y=%.10f  results: fi=%.10f,%.10f",continentMapId, contX, contY,fi,fj))
		return fi,fj
	else
		--library:Debug(string.format("requested data for continentMapId=%d, x=%.10f, y=%.10f",continentMapId, contX, contY))
		assert(false)
		return nil,nil
	end
end

local continentMapIdToPath = {
	[14] = 'Azeroth',
	[13] = 'Kalimdor',
	[466] = 'Outland',
	[485] = 'Northrend',
	[751] = 'Maelstrom',
	[862] = 'Hawaiimainland',
}

function library.MinimapTextureSource(continentMapId, tileI, tileJ)
	--addon:Debug("requesting texture for " .. tostring(continentMapId) .. "," .. tileI .. "," .. tileJ)
	local sContinent = continentMapIdToPath[continentMapId]
	if not sContinent then
		assert(false, "continentMapId " .. continentMapId .. " not known")
	end
	local texPath = string.format([=[World\Minimaps\%s\map%02d_%02d.blp]=], sContinent,tileI, tileJ)
	return texPath
end
